package de.fraunhofer.iem.dataprovider.tools.ort.service

import de.fraunhofer.iem.dataprovider.configuration.OpenCodeApiProperties
import de.fraunhofer.iem.dataprovider.tool.dto.CreateToolDto
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.tool.service.ToolService
import de.fraunhofer.iem.dataprovider.tools.ort.json.DataJson
import de.fraunhofer.iem.dataprovider.tools.ort.json.ResultJson
import io.mockk.every
import io.mockk.mockk
import io.mockk.spyk
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.runTest
import org.junit.jupiter.api.Test

class OrtServiceTest {

    private fun createOrtService(): OrtService {
        val openCodeApiProperties = OpenCodeApiProperties("/localhost", "/ort")
        val toolService = mockk<ToolService>()
        every { toolService.findOrCreateTool(CreateToolDto("ORT")) } returns ToolEntity(name = "ORT")
        return OrtService(openCodeApiProperties, toolService)
    }

    @OptIn(ExperimentalCoroutinesApi::class)
    @Test
    fun getOrtResultsEmpty() = runTest {
        val ortService = spyk(createOrtService(), recordPrivateCalls = true)
        every { ortService invoke "queryOrtApi" withArguments listOf(100.toLong()) } returns emptyList<ResultJson>()
        val ortResults = ortService.getOrtResults(100)
        assert(ortResults.isEmpty())
    }

    @OptIn(ExperimentalCoroutinesApi::class)
    @Test
    fun getOrtResults() = runTest {
        val ortService = spyk(createOrtService(), recordPrivateCalls = true)
        val resultJson = listOf(
            ResultJson(
                createdAt = "",
                cveId = "",
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = "",
                severity = null,
                updatedAt = "",
                version = ""
            ),
            ResultJson(
                createdAt = "",
                cveId = "",
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = null,
                severity = 0.0,
                updatedAt = "",
                version = ""
            ),
            ResultJson(
                createdAt = "",
                cveId = null,
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = "",
                severity = 0.0,
                updatedAt = "",
                version = ""
            ),
            ResultJson(
                createdAt = null,
                cveId = "7",
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = "",
                severity = 0.0,
                updatedAt = "",
                version = ""
            ),
            ResultJson(
                createdAt = "",
                cveId = "11",
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = "Test",
                severity = 0.0,
                updatedAt = "",
                version = ""
            ),
            ResultJson(
                createdAt = "",
                cveId = "42",
                data = DataJson(id = "", description = "", references = emptyList(), summary = ""),
                id = "",
                method = "",
                packageName = "",
                severity = 0.0,
                updatedAt = "",
                version = ""
            ),
        )
        every { ortService invoke "queryOrtApi" withArguments listOf(100.toLong()) } returns resultJson
        val ortResults = ortService.getOrtResults(100)
        assert(ortResults.size == 3)
        assert(ortResults[0].severity == 0.0)
        assert(ortResults[1].packageName == "Test")
        assert(ortResults[0].cveIdentifier == "7")
    }

    @OptIn(ExperimentalCoroutinesApi::class)
    @Test
    fun getOrtResultsFailingApiRequest() = runTest {
        val ortService = createOrtService()

        val ortResults = ortService.getOrtResults(100)
        assert(ortResults.isEmpty())
    }

}
